import { MigrationInterface, QueryRunner, Table, TableForeignKey } from "typeorm";
import { DB_TYPE } from "../env.defaults";

export class Calendar1729947763295 implements MigrationInterface {
  name = "Calendar1729947763295";

  public async up(queryRunner: QueryRunner): Promise<void> {
    const variableType_int = DB_TYPE == "mysql" ? "int" : "integer";

    await queryRunner.createTable(
      new Table({
        name: "calendar_type",
        columns: [
          { name: "id", type: variableType_int, isPrimary: true, isGenerated: true, generationStrategy: "increment" },
          { name: "type", type: "varchar", length: "255", isNullable: false },
          { name: "nscdr", type: "tinyint", isNullable: false },
          { name: "color", type: "varchar", length: "255", isNullable: false },
        ],
      })
    );

    await queryRunner.createTable(
      new Table({
        name: "calendar",
        columns: [
          { name: "id", type: variableType_int, isPrimary: true, isGenerated: true, generationStrategy: "increment" },
          { name: "date", type: "date", isNullable: false },
          { name: "starttime", type: "timestamp", isNullable: true },
          { name: "endtime", type: "datetime", isNullable: true },
          { name: "title", type: "varchar", length: "255", isNullable: false },
          { name: "content", type: "text", isNullable: true },
          { name: "typeId", type: variableType_int, isNullable: false },
        ],
      })
    );

    await queryRunner.createForeignKey(
      "calendar",
      new TableForeignKey({
        columnNames: ["typeId"],
        referencedColumnNames: ["id"],
        referencedTableName: "calendar_type",
        onDelete: "RESTRICT",
        onUpdate: "RESTRICT",
      })
    );
  }

  public async down(queryRunner: QueryRunner): Promise<void> {
    const table = await queryRunner.getTable("calendar");
    const foreignKey = table.foreignKeys.find((fk) => fk.columnNames.indexOf("typeId") !== -1);
    await queryRunner.dropForeignKey("calendar", foreignKey);

    await queryRunner.dropTable("calendar");
    await queryRunner.dropTable("calendar_type");
  }
}
